unit chartprop;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, ComCtrls,
  StdCtrls, ButtonPanel, ColorBox, Spin, TAGraph, TASeries, Types, LCLType,
  FPCanvas, typinfo, TATypes, TAChartAxisUtils, TAChartAxis, TALegend;

type

  { TChartPropForm }

  TChartPropForm = class(TForm)
    cbLegend: TCheckBox;
    cbLegendFontColor: TColorBox;
    cbLegendBackgroundColor: TColorBox;
    cbChartColor: TColorBox;
    cbChartFrameColor: TColorBox;
    Label19: TLabel;
    Label20: TLabel;
    Label21: TLabel;
    Label22: TLabel;
    SetFontBtn: TButton;
    cbPointerColor: TColorBox;
    cbPointerBrushColor: TColorBox;
    cbPointerStyle: TComboBox;
    cbShowLine: TCheckBox;
    cbLinesList: TComboBox;
    cbShowPoints: TCheckBox;
    cbLineColor: TColorBox;
    cbFontColor: TColorBox;
    cbBackgroundColor: TColorBox;
    ColorBtn: TButton;
    cbAxisList: TComboBox;
    cbAlignmentList: TComboBox;
    cbLegendAlignment: TComboBox;
    FontDialog: TFontDialog;
    FontSizeEdit: TEdit;
    gbMarks: TGroupBox;
    Label10: TLabel;
    Label11: TLabel;
    Label12: TLabel;
    Label13: TLabel;
    Label14: TLabel;
    Label15: TLabel;
    Label16: TLabel;
    Label17: TLabel;
    Label18: TLabel;
    Label8: TLabel;
    Label9: TLabel;
    PointerColorBtn: TButton;
    ColorDialog: TColorDialog;
    cbLineStyle: TComboBox;
    gbLine: TGroupBox;
    gbPointer: TGroupBox;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    Label7: TLabel;
    pcChart: TPageControl;
    PointerBrushColorBtn: TButton;
    spLegendColumns: TSpinEdit;
    seAxisPosition: TSpinEdit;
    spPointerHeight: TSpinEdit;
    spLineWidth: TSpinEdit;
    spPointerWidth: TSpinEdit;
    tsChart: TTabSheet;
    tsLegend: TTabSheet;
    tsAxis: TTabSheet;
    tsSeries: TTabSheet;
    _TChart: TChart;
    procedure cbAlignmentListChange(Sender: TObject);
    procedure cbAxisListChange(Sender: TObject);
    procedure cbBackgroundColorChange(Sender: TObject);
    procedure cbChartColorChange(Sender: TObject);
    procedure cbChartFrameColorChange(Sender: TObject);
    procedure cbFontColorChange(Sender: TObject);
    procedure cbLegendAlignmentChange(Sender: TObject);
    procedure cbLegendBackgroundColorChange(Sender: TObject);
    procedure cbLegendChange(Sender: TObject);

    procedure cbPointerBrushColorChange(Sender: TObject);
    procedure cbLineStyleChange(Sender: TObject);
    procedure cbLineStyleDrawItem(Control: TWinControl; Index: Integer;
      ARect: TRect; State: TOwnerDrawState);
    procedure cbPointerColorChange(Sender: TObject);
    procedure cbPointerStyleChange(Sender: TObject);
    procedure cbShowLineChange(Sender: TObject);
    procedure cbShowPointsChange(Sender: TObject);
    procedure cbLegendFontColorChange(Sender: TObject);
    procedure ColorBtnClick(Sender: TObject);
    procedure cbLineColorChange(Sender: TObject);
    procedure cbLinesListChange(Sender: TObject);
    procedure FontSizeEditChange(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure FormClose(Sender: TObject; var CloseAction: TCloseAction);
    procedure FormCreate(Sender: TObject);
    procedure PointerBrushColorBtnClick(Sender: TObject);
    procedure PointerColorBtnClick(Sender: TObject);
    procedure seAxisPositionChange(Sender: TObject);
    procedure SetFontBtnClick(Sender: TObject);
    procedure spLegendColumnsChange(Sender: TObject);
    procedure spPointerHeightChange(Sender: TObject);
    procedure spPointerWidthChange(Sender: TObject);
    procedure spLineWidthChange(Sender: TObject);
  private
    { private declarations }
  public
        { public declarations }
  end;

resourcestring

  LeftAxisStr = 'Left';
  RightAxisStr = 'Rigth';
  BottomAxisStr = 'Bottom';
  TopAxisStr = 'Top';
  LegendAlignmentTopLeftStr = 'TopLeft';
  LegendAlignmentCenterLeftStr =  'CenterLeft';
  LegendAlignmentBottomLeftStr = 'BottomLeft';
  LegendAlignmentTopCenterStr =    'TopCenter';
  LegendAlignmentBottomCenterStr = 'BottomCenter';
  LegendAlignmentTopRightStr =     'TopRight';
  LegendAlignmentCenterRightStr =  'CenterRight';
  LegendAlignmentBottomRightStr =  'BottomRight';

var
  ChartPropForm: TChartPropForm;


implementation

{$R *.lfm}

{ TChartPropForm }

procedure TChartPropForm.FormCreate(Sender: TObject);
var
  i: integer;
begin
  for i:= Ord(Low(TSeriesPointerStyle)) to Ord(High(TSeriesPointerStyle)) do
    cbPointerStyle.Items.Add(GetEnumName(TypeInfo(TSeriesPointerStyle),i));

  cbAlignmentList.Items.Clear;
  cbAlignmentList.Items.Add(LeftAxisStr);
  cbAlignmentList.Items.Add(TopAxisStr);
  cbAlignmentList.Items.Add(RightAxisStr);
  cbAlignmentList.Items.Add(BottomAxisStr);

  cbLegendAlignment.Items.Clear;
  cbLegendAlignment.Items.Add(LegendAlignmentTopLeftStr);
  cbLegendAlignment.Items.Add(LegendAlignmentCenterLeftStr);
  cbLegendAlignment.Items.Add(LegendAlignmentBottomLeftStr);
  cbLegendAlignment.Items.Add(LegendAlignmentTopCenterStr);
  cbLegendAlignment.Items.Add(LegendAlignmentBottomCenterStr);
  cbLegendAlignment.Items.Add(LegendAlignmentTopRightStr);
  cbLegendAlignment.Items.Add(LegendAlignmentCenterRightStr);
  cbLegendAlignment.Items.Add(LegendAlignmentBottomRightStr);
end;

procedure TChartPropForm.PointerBrushColorBtnClick(Sender: TObject);
begin
  if cbLinesList.ItemIndex > -1 then
    if Assigned(_TChart.Series.Items[cbLinesList.ItemIndex]) then
      if ColorDialog.Execute then begin
        cbPointerBrushColor.Items.InsertObject(0,'$' + IntToHex(ColorDialog.Color,8),TObject(PtrInt(ColorDialog.Color)));
        cbPointerBrushColor.Selected :=ColorDialog.Color;
        TLineSeries(_TChart.Series.Items[cbLinesList.ItemIndex]).Pointer.Brush.Color := cbPointerBrushColor.Selected;
    end;
end;

procedure TChartPropForm.PointerColorBtnClick(Sender: TObject);
begin
  if cbLinesList.ItemIndex > -1 then
    if Assigned(_TChart.Series.Items[cbLinesList.ItemIndex]) then
      if ColorDialog.Execute then begin
        cbPointerColor.Items.InsertObject(0,'$' + IntToHex(ColorDialog.Color,8),TObject(PtrInt(ColorDialog.Color)));
        cbPointerColor.Selected :=ColorDialog.Color;
        TLineSeries(_TChart.Series.Items[cbLinesList.ItemIndex]).Pointer.Pen.Color := cbPointerColor.Selected;
    end;
end;

procedure TChartPropForm.seAxisPositionChange(Sender: TObject);
begin
  if cbAxisList.ItemIndex > -1 then
    if Assigned(_TChart) then
     TChartAxis(_TChart.AxisList.Items[cbAxisList.ItemIndex]).Position := seAxisPosition.Value;
end;

procedure TChartPropForm.SetFontBtnClick(Sender: TObject);
begin
  if FontDialog.Execute then
    if Assigned(_TChart) then _TChart.Legend.Font := FontDialog.Font;
end;

procedure TChartPropForm.spPointerHeightChange(Sender: TObject);
begin
  if cbLinesList.ItemIndex > -1 then
    if Assigned(_TChart.Series.Items[cbLinesList.ItemIndex]) then
      TLineSeries(_TChart.Series.Items[cbLinesList.ItemIndex]).Pointer.VertSize := spPointerHeight.Value;
end;

procedure TChartPropForm.spPointerWidthChange(Sender: TObject);
begin
  if cbLinesList.ItemIndex > -1 then
    if Assigned(_TChart.Series.Items[cbLinesList.ItemIndex]) then
      TLineSeries(_TChart.Series.Items[cbLinesList.ItemIndex]).Pointer.HorizSize := spPointerWidth.Value;
end;

procedure TChartPropForm.spLineWidthChange(Sender: TObject);
begin
  if cbLinesList.ItemIndex > -1 then
    if Assigned(_TChart.Series.Items[cbLinesList.ItemIndex]) then
      TLineSeries(_TChart.Series.Items[cbLinesList.ItemIndex]).LinePen.Width := spLineWidth.Value;
end;

procedure TChartPropForm.FormActivate(Sender: TObject);
var
  i:integer;
begin
  if Assigned(_TChart) then begin
    for i:= 0 to _TChart.SeriesCount-1 do
      cbLinesList.Items.Add(TLineSeries(_TChart.Series.Items[i]).Title);
    with _TChart.Legend do begin
       spLegendColumns.Value := ColumnCount;
       cbLegendAlignment.ItemIndex := Ord(Alignment);
       FontDialog.Font := Font;
       cbLegendFontColor.Selected := Font.Color;
       cbLegendBackgroundColor.Selected := BackgroundBrush.Color;
       cbLegend.Checked := Visible;
    end;
    with _TChart do begin
       cbChartColor.Selected := Color;
       cbChartFrameColor.Selected := BackColor;
    end;
  end;

  CbLineColor.ItemIndex := -1;
  cbFontColor.ItemIndex:= -1;
end;

procedure TChartPropForm.FormClose(Sender: TObject;
  var CloseAction: TCloseAction);
begin
  CloseAction := caFree;
end;

procedure TChartPropForm.cbLinesListChange(Sender: TObject);
begin
  if cbLinesList.ItemIndex <= -1 then Exit;
  if NOT(Assigned(_TChart.Series.Items[cbLinesList.ItemIndex])) then Exit;
  //line
  with TLineSeries(_TChart.Series.Items[cbLinesList.ItemIndex]).LinePen do begin
    cbLineColor.Selected := Color;
    if cbLineColor.ItemIndex = -1 then begin
      cbLineColor.Items.InsertObject(0,'$'+IntToHex(Color,8),TObject(PtrInt(Color)));
      cbLineColor.ItemIndex := 0;
    end;

    cbLineStyle.ItemIndex := ord(Style);
    spLineWidth.Value := Width;
    cbShowLine.Checked := TLineSeries(_TChart.Series.Items[cbLinesList.ItemIndex]).ShowLines;
  end;
  //pointer
  with TLineSeries(_TChart.Series.Items[cbLinesList.ItemIndex]).Pointer do begin
     cbPointerColor.Selected := Pen.Color;
     if cbPointerColor.ItemIndex = -1 then begin
      cbPointerColor.Items.InsertObject(0,'$'+IntToHex(Color,8),TObject(PtrInt(Pen.Color)));
      cbPointerColor.ItemIndex := 0;
     end;

     cbPointerBrushColor.Selected := Brush.Color;
     if cbPointerBrushColor.ItemIndex = -1 then begin
      cbPointerBrushColor.Items.InsertObject(0,'$'+IntToHex(Color,8),TObject(PtrInt(Pen.Color)));
      cbPointerBrushColor.ItemIndex := 0;
     end;
     cbPointerStyle.ItemIndex := Ord(Style);
     spPointerWidth.Value := HorizSize;
     spPointerHeight.Value  := VertSize;
     cbShowPoints.Checked := TLineSeries(_TChart.Series.Items[cbLinesList.ItemIndex]).ShowPoints;
  end;
end;

procedure TChartPropForm.FontSizeEditChange(Sender: TObject);
begin
   if cbAxisList.ItemIndex >-1 then
   if Assigned(_TChart) then
     TChartAxis(_TChart.AxisList.Items[cbAxisList.ItemIndex]).Marks.LabelFont.Size := StrToIntDef(FontSizeEdit.Text,0);
end;

procedure TChartPropForm.cbLineColorChange(Sender: TObject);
begin
  if cbLineColor.Items.Count > 20 then cbLineColor.Items.Delete(0);
  if cbLinesList.ItemIndex > -1 then
    if Assigned(_TChart.Series.Items[cbLinesList.ItemIndex]) then
      TLineSeries(_TChart.Series.Items[cbLinesList.ItemIndex]).LinePen.Color := cbLineColor.Selected;
end;

procedure TChartPropForm.ColorBtnClick(Sender: TObject);
begin
  if cbLinesList.ItemIndex > -1 then
    if Assigned(_TChart.Series.Items[cbLinesList.ItemIndex]) then
      if ColorDialog.Execute then begin
        cbLineColor.Items.InsertObject(0,'$' + IntToHex(ColorDialog.Color,8),TObject(PtrInt(ColorDialog.Color)));
        cbLineColor.Selected := ColorDialog.Color;
        TLineSeries(_TChart.Series.Items[cbLinesList.ItemIndex]).LinePen.Color := cbLineColor.Selected;
    end;
end;

procedure TChartPropForm.cbShowPointsChange(Sender: TObject);
begin
  if cbLinesList.ItemIndex > -1 then
    if Assigned(_TChart.Series.Items[cbLinesList.ItemIndex]) then
      TLineSeries(_TChart.Series.Items[cbLinesList.ItemIndex]).ShowPoints := cbShowPoints.Checked;
end;

procedure TChartPropForm.cbLegendFontColorChange(Sender: TObject);
begin
  if Assigned(_TChart) then
    _TChart.Legend.Font.Color := cbLegendFontColor.Selected;
end;

procedure TChartPropForm.cbShowLineChange(Sender: TObject);
begin
  if cbLinesList.ItemIndex > -1 then
    if Assigned(_TChart.Series.Items[cbLinesList.ItemIndex]) then
      TLineSeries(_TChart.Series.Items[cbLinesList.ItemIndex]).ShowLines := cbShowLine.Checked;
end;

procedure TChartPropForm.cbLineStyleDrawItem(Control: TWinControl;
  Index: Integer; ARect: TRect; State: TOwnerDrawState);
begin
  with (Control as TComboBox).Canvas do begin
       Pen.Style := TFPPenStyle(Index);
       if  odSelected in State then
           Pen.Color := clHighLightText
       else
           Pen.Color := clWindowText;
       FillRect(ARect);
       MoveTo(ARect.Left+8,ARect.Bottom - ((ARect.Bottom - ARect.Top) div 3));
       LineTo(ARect.Right-8, ARect.Bottom - ((ARect.Bottom - ARect.Top) div 3));
   end;
end;

procedure TChartPropForm.cbPointerColorChange(Sender: TObject);
begin
  if cbLinesList.Items.Count > 20 then cbLinesList.Items.Delete(0);
  if cbLinesList.ItemIndex > -1 then
    if Assigned(_TChart.Series.Items[cbLinesList.ItemIndex]) then
      TLineSeries(_TChart.Series.Items[cbLinesList.ItemIndex]).Pointer.Pen.Color := cbPointerColor.Selected;
end;

procedure TChartPropForm.cbPointerStyleChange(Sender: TObject);
begin
  if cbLinesList.ItemIndex > -1 then
    if Assigned(_TChart.Series.Items[cbLinesList.ItemIndex]) then
      TLineSeries(_TChart.Series.Items[cbLinesList.ItemIndex]).Pointer.Style := TSeriesPointerStyle(cbPointerStyle.ItemIndex);
end;

procedure TChartPropForm.cbLineStyleChange(Sender: TObject);
begin
  if cbLinesList.ItemIndex > -1 then
    if Assigned(_TChart.Series.Items[cbLinesList.ItemIndex]) then
      TLineSeries(_TChart.Series.Items[cbLinesList.ItemIndex]).LinePen.Style := TFPPenStyle(cbLineStyle.ItemIndex);
end;

procedure TChartPropForm.cbPointerBrushColorChange(Sender: TObject);
begin
   if cbPointerBrushColor.Items.Count > 20 then cbPointerBrushColor.Items.Delete(0);
   if cbLinesList.ItemIndex > -1 then
    if Assigned(_TChart.Series.Items[cbLinesList.ItemIndex]) then
      TLineSeries(_TChart.Series.Items[cbLinesList.ItemIndex]).Pointer.Brush.Color := cbPointerBrushColor.Selected;
end;

procedure TChartPropForm.cbAlignmentListChange(Sender: TObject);
begin
  if cbAxisList.ItemIndex >-1 then
   if Assigned(_TChart) then
    TChartBasicAxis(_TChart.AxisList.Items[cbAxisList.ItemIndex]).Alignment := TChartAxisAlignment(cbAlignmentList.ItemIndex);
end;

procedure TChartPropForm.cbAxisListChange(Sender: TObject);
begin
  if cbAxisList.ItemIndex >-1 then
   if Assigned(_TChart) then
    with TChartAxis(_TChart.AxisList.Items[cbAxisList.ItemIndex]) do begin
       cbAlignmentList.ItemIndex := Ord(Alignment);
       seAxisPosition.Value := Position;
       FontSizeEdit.Text := IntToStr(Marks.LabelFont.Size);
       cbFontColor.Selected := Marks.LabelFont.Color;

       if cbFontColor.ItemIndex = -1 then cbFontColor.Items.InsertObject(0,'Custom...',TObject(PtrInt(Marks.LabelFont.Color)));

       cbBackgroundColor.Selected := Marks.LabelBrush.Color;
       if cbBackgroundColor.ItemIndex = -1 then cbBackgroundColor.Items.InsertObject(0,'Custom...',TObject(PtrInt(Marks.LabelFont.Color)));
   end;
end;

procedure TChartPropForm.cbBackgroundColorChange(Sender: TObject);
begin
  if cbAxisList.ItemIndex >-1 then
   if Assigned(_TChart) then
    TChartAxis(_TChart.AxisList.Items[cbAxisList.ItemIndex]).Marks.LabelBrush.Color := cbBackgroundColor.Selected;
end;

procedure TChartPropForm.cbChartColorChange(Sender: TObject);
begin
  if Assigned(_TChart) then
    _TChart.Color := cbChartColor.Selected;
end;

procedure TChartPropForm.cbChartFrameColorChange(Sender: TObject);
begin
  if Assigned(_TChart) then
    _TChart.BackColor := cbChartFrameColor.Selected;
end;

procedure TChartPropForm.cbFontColorChange(Sender: TObject);
begin
  if cbAxisList.ItemIndex >-1 then
   if Assigned(_TChart) then
    TChartAxis(_TChart.AxisList.Items[cbAxisList.ItemIndex]).Marks.LabelFont.Color := cbFontColor.Selected;
end;

procedure TChartPropForm.cbLegendAlignmentChange(Sender: TObject);
begin
  if Assigned(_TChart) then _TChart.Legend.Alignment := TLegendAlignment(cbLegendAlignment.ItemIndex);
end;

procedure TChartPropForm.cbLegendBackgroundColorChange(Sender: TObject);
begin
  if Assigned(_TChart) then _TChart.Legend.BackgroundBrush.Color :=  cbLegendBackgroundColor.Selected;
end;

procedure TChartPropForm.cbLegendChange(Sender: TObject);
begin
  if Assigned(_TChart) then _TChart.Legend.Visible := cbLegend.Checked;
end;

procedure TChartPropForm.spLegendColumnsChange(Sender: TObject);
begin
  if Assigned(_TChart) then _TChart.Legend.ColumnCount := spLegendColumns.Value;
end;

end.

