unit measfile;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, dialogs, StrUtils, dateutils, Grids, TAGraph, TASeries,
  loadsaveconfig, ComCtrls, Forms, FileUtil,syncobjs;

type

  { TReadMeasFileThread }

  TReadMeasFileThread = class(TThread)
  private
    FFileName: string;
    FIbChart: TChart;
    FIbGrid: TStringGrid;
    FIrChart: TChart;
    FIrGrid: TStringGrid;
    FProgressBar: TProgressBar;
    FReadedBytes:int64;
    FStatusPanel: TStatusPanel;

    FTemperatureChart: TChart;
    FTemperatureGrid: TStringGrid;
    FVoltageChart: TChart;
    FVoltageGrid: TStringGrid;
    FFileSize: int64;
    FAString: string;
    FRowN: integer;
    procedure SetFileName(AValue: string);
    procedure SetIbChart(AValue: TChart);
    procedure SetIbGrid(AValue: TStringGrid);
    procedure SetIrChart(AValue: TChart);
    procedure SetIrGrid(AValue: TStringGrid);
    procedure SetProgressBar(AValue: TProgressBar);
    procedure SetStatusPanel(AValue: TStatusPanel);

    procedure SetTemperatureChart(AValue: TChart);
    procedure SetTemperatureGrid(AValue: TStringGrid);
    procedure SetVoltageChart(AValue: TChart);
    procedure SetVoltageGrid(AValue: TStringGrid);
    procedure UpdateProgress;
  protected
    procedure Execute; override;
  public
    cs:TCriticalSection;
    property IrGrid          : TStringGrid read FIrGrid write SetIrGrid;
    property IbGrid          : TStringGrid read FIbGrid write SetIbGrid;
    property TemperatureGrid : TStringGrid read FTemperatureGrid write SetTemperatureGrid;
    property VoltageGrid     : TStringGrid read FVoltageGrid write SetVoltageGrid;
    property IrChart           : TChart read FIrChart write SetIrChart;
    property IbChart           : TChart read FIbChart write SetIbChart;
    property TemperatureChart  : TChart read FTemperatureChart write SetTemperatureChart;
    property VoltageChart      : TChart read FVoltageChart write SetVoltageChart;
    property ProgressBar: TProgressBar read FProgressBar write SetProgressBar;
    property StatusPanel: TStatusPanel read FStatusPanel write SetStatusPanel;

    property FileName: string read FFileName write SetFileName;
    constructor Create(CreateSuspended: boolean);
    destructor Destroy;override;
  end;


procedure CreateMeasFileHeader(FileName:string; StartDate,EndDate:TDateTime;
    Note:string; StabilityType:string);
//vraci pocet prectenych bytu
function ReadMeasFileHeader(const FileName:string; out StartDate,EndDate:TDateTime;
    out Note:string; out StabilityType:string): longword;

procedure CreateMeasFileTableTitles(const FileName:string; Titles: TStrings);

function ReadMeasFileTableTitles(const FileName:string; Titles: TStrings): longword;

procedure ReadMeasFileTableTitlesToCharts(const FileName:string; _IrChart, _IbChart, _TemperatureChart: TChart);

procedure WriteDataToMeasFile(const FileName: string;
    const IrArr, IbArr, TArr: array of double; const Ur, Ub: double);

//vraci pocet prectenych bytu
function ReadDataFromMeasFileToGrids(const FileName: string;
    const _IrGrid, _IbGrid, _TemperatureGrid, _VoltageGrid :TStringGrid;
    const _IrChart, _IbChart, _TemperatureChart, _VoltageChart: TChart): longword;

procedure WriteDataToMeasFileFromGrids(const FileName: string;
    const _IrGrid, _IbGrid, _TemperatureGrid, _VoltageGrid :TStringGrid);

resourcestring
  FileOpeningInProgressStr = 'File opening in progress...';
const
  DateStr = 'Date';
  TimeStr = 'Time';
  IrStr = 'Ir, [mA]';
  IdStr = 'Id, [mA]';
  UrStr = 'Ur, [V]';
  UdStr = 'Ud, [V]';
  TemperatureStr = 'T, [°C]';
  StartTestStr = 'Test started';
  EndTestStr = 'Test ended';
  NoteStr = 'Note';
  StabilityStr = 'Stability';


implementation

procedure CreateMeasFileHeader(FileName: string; StartDate, EndDate:TDateTime;
  Note: string; StabilityType: string);
var
  FileHeader:TStringList;
begin
  FileHeader :=TStringList.Create;
  try
    FileHeader.Add(StartTestStr+#9+DateTimeToStr(StartDate));
    FileHeader.Add(EndTestStr+#9+DateTimeToStr(EndDate));
    FileHeader.Add(NoteStr+#9+Note);
    FileHeader.Add(StabilityStr+#9+StabilityType);
    FileHeader.SaveToFile(FileName);
  finally
    FileHeader.Free;
  end;
end;

function ReadMeasFileHeader(const FileName: string; out StartDate,
  EndDate: TDateTime; out Note: string; out StabilityType: string): longword;
var

  AFile: TextFile;
  AString: string;
begin
  Result := 0;

  assignFile(AFile, FileName);
  try
    reset(AFile);

    readln(AFile, AString);
    Result := Result + Length(AString);
    StartDate := StrToDateTime(ExtractDelimited(2,AString,[#9]));


    readln(AFile, AString);
    EndDate := StrToDateTime(ExtractDelimited(2,AString,[#9]));
    Result := Result + Length(AString);

    readln(AFile, AString);
    Note := ExtractDelimited(2,AString,[#9]);
    Result := Result + Length(AString);

    readln(AFile, AString);
    StabilityType := ExtractDelimited(2,AString,[#9]);
    Result := Result + Length(AString);
  finally
    CloseFile(AFile);
  end;
end;

procedure CreateMeasFileTableTitles(const FileName: string; Titles: TStrings);
var
  AFile: TextFile;
  i: integer;
begin
  assignFile(AFile, FileName);
  try
    append(AFile);
    write(AFile,DateStr + #9 + TimeStr + #9);
    for i:= 1 to Titles.Count-1 do begin
      write(AFile, Titles.Strings[i] + ' ' + IrStr + #9);
      write(AFile, Titles.Strings[i] + ' ' + IdStr + #9);
      write(AFile, Titles.Strings[i] + ' ' + TemperatureStr + #9);
    end;
    write(AFile,UrStr + #9 + UdStr);
  finally
    closeFile(AFile);
  end;
end;

function ReadMeasFileTableTitles(const FileName: string; Titles: TStrings
  ): longword;
var
  AFile: TextFile;
  AString: string;
  AStringList: TStringList;
  i: integer;
begin
  Result:= 0;
  assignFile(AFile,FileName);
  AStringList := TStringList.Create;
  try
    reset(AFile);
    readln(AFile);
    readln(AFile);
    readln(AFile);
    readln(AFile);
    readln(AFile,AString);
    Result := Length(AString);

    AString := StringReplace(AString,DateStr+#9,'',[rfReplaceAll]);
    AString := StringReplace(AString,TimeStr+#9,'',[rfReplaceAll]);
    AString := StringReplace(AString,IrStr,'',[rfReplaceAll]);
    AString := StringReplace(AString,IdStr,'',[rfReplaceAll]);
    AString := StringReplace(AString,TemperatureStr,'',[rfReplaceAll]);
    AString := StringReplace(AString,UrStr+#9,'',[rfReplaceAll]);
    AString := StringReplace(AString,UdStr,'',[rfReplaceAll]);
    for i:= 1 to WordCount(AString,[#9]) do AStringList.Add(ExtractDelimited(i,AString,[#9]));
    //remove duplicates
    for i:= AStringList.Count-1 downto 1 do
      if AStringList.Strings[i] = AStringList.Strings[i-1] then AStringList.Delete(i);
    //add dummy for Col title
    AStringList.Insert(0,'');
    Titles.Assign(AStringList);

  finally
    AStringList.Create;
    closeFile(AFile);
  end;
end;

procedure ReadMeasFileTableTitlesToCharts(const FileName: string; _IrChart, _IbChart,
  _TemperatureChart: TChart);
var
  AFile: TextFile;
  AString: string;
  AStringList: TStringList;
  i: integer;
begin
  assignFile(AFile,FileName);
  AStringList := TStringList.Create;
  try
    reset(AFile);
    readln(AFile);
    readln(AFile);
    readln(AFile);
    readln(AFile);
    readln(AFile,AString);

    AString := StringReplace(AString,DateStr+#9,'',[rfReplaceAll]);
    AString := StringReplace(AString,TimeStr+#9,'',[rfReplaceAll]);
    AString := StringReplace(AString,IrStr,'',[rfReplaceAll]);
    AString := StringReplace(AString,IdStr,'',[rfReplaceAll]);
    AString := StringReplace(AString,TemperatureStr,'',[rfReplaceAll]);
    AString := StringReplace(AString,UrStr+#9,'',[rfReplaceAll]);
    AString := StringReplace(AString,UdStr,'',[rfReplaceAll]);
    for i:= 1 to WordCount(AString,[#9]) do AStringList.Add(ExtractDelimited(i,AString,[#9]));
    //remove duplicates
    for i:= AStringList.Count-1 downto 1 do
      if AStringList.Strings[i] = AStringList.Strings[i-1] then AStringList.Delete(i);

    for i:= 0 to AStringList.Count-1 do begin
      TLineSeries(_IrChart.Series.Items[i]).Title := AStringList.Strings[i];
      TLineSeries(_IbChart.Series.Items[i]).Title := AStringList.Strings[i];
      TLineSeries(_TemperatureChart.Series.Items[i]).Title := AStringList.Strings[i];
    end;
  finally
    AStringList.Create;
    closeFile(AFile);
  end;
end;

procedure WriteDataToMeasFile(const FileName: string;
  const IrArr, IbArr, TArr: array of double; const Ur, Ub: double);
var
  i: integer;
  AFile: TextFile;
begin
  assignFile(AFile, FileName);
  try
    append(AFile);
    writeln(AFile);
    write(AFile,DateToStr(DateOf(Date)) + #9 + TimeToStr(Time) + #9);
    for i:= Low(IrArr) to High(IrArr) do
      write(AFile, FloatToStrF(IrArr[i],ffFixed,8,2) + #9 +
        FloatToStrF(IbArr[i],ffFixed,8,2) + #9 + FloatToStrF(TArr[i],ffFixed,8,0) + #9);
    write(AFile, FloatToStrF(Ur,ffFixed,8,0) + #9 + FloatToStrF(Ub,ffFixed,8,0));
  finally
    CloseFile(AFile);
  end;
end;

function ReadDataFromMeasFileToGrids(const FileName: string; const _IrGrid,
  _IbGrid, _TemperatureGrid, _VoltageGrid: TStringGrid; const _IrChart,
  _IbChart, _TemperatureChart, _VoltageChart: TChart): longword;
var
  AFile: TextFile;
  AString: string;
  i: integer;
  RowN: integer = 0;
begin
  Result := 0;
  for i:= 0 to _IrChart.SeriesCount-1 do TLineSeries(_IrChart.Series.Items[i]).Clear;
  for i:= 0 to _IbChart.SeriesCount-1 do TLineSeries(_IbChart.Series.Items[i]).Clear;
  for i:= 0 to _TemperatureChart.SeriesCount-1 do TLineSeries(_TemperatureChart.Series.Items[i]).Clear;
  for i:= 0 to _VoltageChart.SeriesCount-1 do TLineSeries(_VoltageChart.Series.Items[i]).Clear;

  assignFile(AFile, FileName);
  try
    reset(AFile);
    readln(AFile);
    readln(AFile);
    readln(AFile);
    readln(AFile);
    readln(AFile);
    while NOT(EOF(AFile)) do begin
      inc(RowN);
      readln(AFile, AString);
      _IrGrid.RowCount := _IrGrid.RowCount + 1;
      _IbGrid.RowCount := _IbGrid.RowCount + 1;
      _TemperatureGrid.RowCount := _TemperatureGrid.RowCount + 1;
      _VoltageGrid.RowCount := _VoltageGrid.RowCount + 1;

      _IrGrid.Cells[1,RowN] := ExtractDelimited(1,AString,[#9]);
      _IrGrid.Cells[2,RowN] := ExtractDelimited(2,AString,[#9]);
      _IbGrid.Cells[1,RowN] := ExtractDelimited(1,AString,[#9]);
      _IbGrid.Cells[2,RowN] := ExtractDelimited(2,AString,[#9]);
      _TemperatureGrid.Cells[1,RowN] := ExtractDelimited(1,AString,[#9]);
      _TemperatureGrid.Cells[2,RowN] := ExtractDelimited(2,AString,[#9]);
      _VoltageGrid.Cells[1,RowN] := ExtractDelimited(1,AString,[#9]);
      _VoltageGrid.Cells[2,RowN] := ExtractDelimited(2,AString,[#9]);
      for i:= 1 to defaultDeviceCount do begin
        _IrGrid.Cells[i+2,RowN] := ExtractDelimited(i*3, AString,[#9]);
        _IbGrid.Cells[i+2,RowN] := ExtractDelimited(i*3+1, AString,[#9]);
        _TemperatureGrid.Cells[i+2,RowN] := ExtractDelimited(i*3+2, AString,[#9]);

        TLineSeries(_IrChart.Series.Items[i-1]).AddXY(StrToDateTime(ExtractDelimited(1,AString,[#9])+' '+ExtractDelimited(2,AString,[#9])),StrToFloat(ExtractDelimited(i*3, AString,[#9])));
        TLineSeries(_IbChart.Series.Items[i-1]).AddXY(StrToDateTime(ExtractDelimited(1,AString,[#9])+' '+ExtractDelimited(2,AString,[#9])),StrToFloat(ExtractDelimited(i*3+1, AString,[#9])));
        TLineSeries(_TemperatureChart.Series.Items[i-1]).AddXY(StrToDateTime(ExtractDelimited(1,AString,[#9])+' '+ExtractDelimited(2,AString,[#9])),StrToFloat(ExtractDelimited(i*3+2, AString,[#9])));

      end;
      _VoltageGrid.Cells[3,RowN] := ExtractDelimited(51, AString,[#9]);
      _VoltageGrid.Cells[4,RowN] := ExtractDelimited(52, AString,[#9]);
      TLineSeries(_VoltageChart.Series.Items[0]).AddXY(StrToDateTime(ExtractDelimited(1,AString,[#9])+' '+ExtractDelimited(2,AString,[#9])),StrToFloat(ExtractDelimited(51, AString,[#9])));
      TLineSeries(_VoltageChart.Series.Items[1]).AddXY(StrToDateTime(ExtractDelimited(1,AString,[#9])+' '+ExtractDelimited(2,AString,[#9])),StrToFloat(ExtractDelimited(52, AString,[#9])));

      Result := Result + Length(AString);


    end;
  finally
    CloseFile(AFile);
  end;
end;

procedure WriteDataToMeasFileFromGrids(const FileName: string; const _IrGrid,
  _IbGrid, _TemperatureGrid, _VoltageGrid: TStringGrid);
var
  i,j: integer;
  AFile:TextFile;
begin
  assignFile(AFile, FileName);
  try
    append(AFile);
    for i:= 1 to _IrGrid.RowCount-1 do begin
      writeln(AFile);
      //date nad time
      for j:= 1 to 2 do
          write(AFile, _IrGrid.Cells[j,i] + #9);
      //currents Ir,Ib and temperature value
      for j:= 3 to _IrGrid.ColCount-1 do
        write(AFile, _IrGrid.Cells[j,i] + #9 + _IbGrid.Cells[j,i] + #9 + _TemperatureGrid.Cells[j,i] + #9);
      //voltage
      write(AFile,_VoltageGrid.Cells[3,i] + #9);
      write(AFile,_VoltageGrid.Cells[4,i]);
    end;
  finally
    CloseFile(AFile);
  end;
end;

{ TReadMeasFileThread }

procedure TReadMeasFileThread.SetProgressBar(AValue: TProgressBar);
begin
  if FProgressBar=AValue then Exit;
  FProgressBar:=AValue;
  FProgressBar.Visible := True;
end;

procedure TReadMeasFileThread.SetStatusPanel(AValue: TStatusPanel);
begin
  if FStatusPanel=AValue then Exit;
  FStatusPanel:=AValue;
end;

procedure TReadMeasFileThread.SetTemperatureChart(AValue: TChart);
begin
  if FTemperatureChart=AValue then Exit;
  FTemperatureChart:=AValue;
end;

procedure TReadMeasFileThread.SetTemperatureGrid(AValue: TStringGrid);
begin
  if FTemperatureGrid=AValue then Exit;
  FTemperatureGrid:=AValue;
end;

procedure TReadMeasFileThread.SetVoltageChart(AValue: TChart);
begin
  if FVoltageChart=AValue then Exit;
  FVoltageChart:=AValue;
end;

procedure TReadMeasFileThread.SetVoltageGrid(AValue: TStringGrid);
begin
  if FVoltageGrid=AValue then Exit;
  FVoltageGrid:=AValue;
end;

procedure TReadMeasFileThread.UpdateProgress;
var

  i: integer;

begin
  cs.Enter;
   StatusPanel.Text := FileOpeningInProgressStr;

   if ProgressBar <> Nil then ProgressBar.Position := Round(FReadedBytes/FFileSize * 100);
   IrGrid.RowCount := IrGrid.RowCount + 1;
   IbGrid.RowCount := IbGrid.RowCount + 1;
   TemperatureGrid.RowCount := TemperatureGrid.RowCount + 1;
   VoltageGrid.RowCount := VoltageGrid.RowCount + 1;
   IrGrid.Cells[1,FRowN] := ExtractDelimited(1,FAString,[#9]);
   IrGrid.Cells[2,FRowN] := ExtractDelimited(2,FAString,[#9]);
   IbGrid.Cells[1,FRowN] := ExtractDelimited(1,FAString,[#9]);
   IbGrid.Cells[2,FRowN] := ExtractDelimited(2,FAString,[#9]);
   TemperatureGrid.Cells[1,FRowN] := ExtractDelimited(1,FAString,[#9]);
   TemperatureGrid.Cells[2,FRowN] := ExtractDelimited(2,FAString,[#9]);
   VoltageGrid.Cells[1,FRowN] := ExtractDelimited(1,FAString,[#9]);
   VoltageGrid.Cells[2,FRowN] := ExtractDelimited(2,FAString,[#9]);
   for i:= 1 to defaultDeviceCount do begin
     IrGrid.Cells[i+2,FRowN] := ExtractDelimited(i*3, FAString,[#9]);
     IbGrid.Cells[i+2,FRowN] := ExtractDelimited(i*3+1, FAString,[#9]);
     TemperatureGrid.Cells[i+2,FRowN] := ExtractDelimited(i*3+2, FAString,[#9]);
        TLineSeries(IrChart.Series.Items[i-1]).AddXY(StrToDateTime(ExtractDelimited(1,FAString,[#9])+' '+ExtractDelimited(2,FAString,[#9])),StrToFloat(ExtractDelimited(i*3, FAString,[#9])));
     TLineSeries(IbChart.Series.Items[i-1]).AddXY(StrToDateTime(ExtractDelimited(1,FAString,[#9])+' '+ExtractDelimited(2,FAString,[#9])),StrToFloat(ExtractDelimited(i*3+1, FAString,[#9])));
     TLineSeries(TemperatureChart.Series.Items[i-1]).AddXY(StrToDateTime(ExtractDelimited(1,FAString,[#9])+' '+ExtractDelimited(2,FAString,[#9])),StrToFloat(ExtractDelimited(i*3+2, FAString,[#9])));
      end;
   VoltageGrid.Cells[3,FRowN] := ExtractDelimited(51, FAString,[#9]);
   VoltageGrid.Cells[4,FRowN] := ExtractDelimited(52, FAString,[#9]);
   TLineSeries(VoltageChart.Series.Items[0]).AddXY(StrToDateTime(ExtractDelimited(1,FAString,[#9])+' '+ExtractDelimited(2,FAString,[#9])),StrToFloat(ExtractDelimited(51, FAString,[#9])));
   TLineSeries(VoltageChart.Series.Items[1]).AddXY(StrToDateTime(ExtractDelimited(1,FAString,[#9])+' '+ExtractDelimited(2,FAString,[#9])),StrToFloat(ExtractDelimited(52, FAString,[#9])));
  cs.Leave;
end;

procedure TReadMeasFileThread.SetFileName(AValue: string);
begin
  if FFileName=AValue then Exit;
  FFileName:=AValue;
  if FileExists(FFileName) then FFileSize := FileSize(FFileName);
end;

procedure TReadMeasFileThread.SetIbChart(AValue: TChart);
begin
  if FIbChart=AValue then Exit;
  FIbChart:=AValue;
end;

procedure TReadMeasFileThread.SetIbGrid(AValue: TStringGrid);
begin
  if FIbGrid=AValue then Exit;
  FIbGrid:=AValue;
end;

procedure TReadMeasFileThread.SetIrChart(AValue: TChart);
begin
  if FIrChart=AValue then Exit;
  FIrChart:=AValue;
end;

procedure TReadMeasFileThread.SetIrGrid(AValue: TStringGrid);
begin
  if FIrGrid=AValue then Exit;
  FIrGrid:=AValue;
end;

procedure TReadMeasFileThread.Execute;
var
  AFile: TextFile;
begin
  FReadedBytes := 0;
  FRowN := 0;
  for i:= 0 to IrChart.SeriesCount-1 do TLineSeries(IrChart.Series.Items[i]).Clear;
  for i:= 0 to IbChart.SeriesCount-1 do TLineSeries(IbChart.Series.Items[i]).Clear;
  for i:= 0 to TemperatureChart.SeriesCount-1 do TLineSeries(TemperatureChart.Series.Items[i]).Clear;
  for i:= 0 to VoltageChart.SeriesCount-1 do TLineSeries(VoltageChart.Series.Items[i]).Clear;

  IrGrid.BeginUpdate;
  IbGrid.BeginUpdate;
  TemperatureGrid.BeginUpdate;
  VoltageGrid.BeginUpdate;
  IrChart.DisableRedrawing;
  IbChart.DisableRedrawing;
  TemperatureChart.DisableRedrawing;
  VoltageChart.DisableRedrawing;

  assignFile(AFile, FileName);
  try
      reset(AFile);
      readln(AFile);
      readln(AFile);
      readln(AFile);
      readln(AFile);
      readln(AFile);
      while NOT(EOF(AFile)) and NOT(Terminated) do begin
        inc(FRowN);
        readln(AFile, FAString);
        FReadedBytes := FReadedBytes + Length(FAString);
        Synchronize(@UpdateProgress);
      end;

  finally
    CloseFile(AFile);
    VoltageGrid.EndUpdate();
    IrGrid.EndUpdate();
    IbGrid.EndUpdate();
    TemperatureGrid.EndUpdate();
    IrChart.EnableRedrawing;
    IbChart.EnableRedrawing;
    TemperatureChart.EnableRedrawing;
    VoltageChart.EnableRedrawing;

    Terminate;
  end;
  StatusPanel.Text := '';
end;

constructor TReadMeasFileThread.Create(CreateSuspended: boolean);
begin
  inherited Create(CreateSuspended);
  FreeOnTerminate := True;
  cs :=TCriticalSection.Create;
end;

destructor TReadMeasFileThread.Destroy;
begin
  FProgressBar.Visible := False;
  cs.Free;
  inherited Destroy;
end;


initialization

  FormatSettings.LongDateFormat := 'DD.MM.YYYY';
  FormatSettings.LongTimeFormat := 'hh:nn:ss';

end.

